@@ -45,6 +45,18 @@ def test_f_literals():
     f'x{bar["baz"]:.2}y\n'
 
 a = "Escapes in String \U12345678 \xAB \""
+a = u'''\'''
+'''
+a = u'''\''''
+a = b'\u1234a\xffé\12k\n\g\
+'
+a = rb'\u1234aé\n\g\
+'
+a = b""""""
+a = b"""\""""
+a = 'a' \
+    u'\x12'
+
 "Escapes in Doc/Comment String \u1234 \xAb \"\\"
 ''' Doc/Comment String \N{123} \''''
 """ Doc/Comment String \17 \x1f \n \" \\"""
@@ -80,3 +92,19 @@ u = 1_0.e+3_22j + .2_1e2j + 11_1.j
 aaa.123
 aaa123
 .0x123
+
+# match and case keywords
+foo(
+    # no keyword
+    match if xx else yyy
+)
+match = 3
+match.foo()
+match command.split():
+    case ["quit"]:
+        print("Goodbye!")
+
+match(command.split())
+match(command.split()):
+    case ["quit"]:
+        print("Goodbye!")
