@@ -1,29 +1,12 @@
 /*
-    Copyright (C) 2016 Volker Krause <vkrause@kde.org>
-    Copyright (C) 2018 Dominik Haumann <dhaumann@kde.org>
-    Copyright (C) 2018 Christoph Cullmann <cullmann@kde.org>
+    SPDX-FileCopyrightText: 2016 Volker Krause <vkrause@kde.org>
+    SPDX-FileCopyrightText: 2018 Dominik Haumann <dhaumann@kde.org>
+    SPDX-FileCopyrightText: 2018 Christoph Cullmann <cullmann@kde.org>
+    SPDX-FileCopyrightText: 2020 Jonathan Poelen <jonathan.poelen@gmail.com>
 
-    Permission is hereby granted, free of charge, to any person obtaining
-    a copy of this software and associated documentation files (the
-    "Software"), to deal in the Software without restriction, including
-    without limitation the rights to use, copy, modify, merge, publish,
-    distribute, sublicense, and/or sell copies of the Software, and to
-    permit persons to whom the Software is furnished to do so, subject to
-    the following conditions:
-
-    The above copyright notice and this permission notice shall be included
-    in all copies or substantial portions of the Software.
-
-    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
-    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
-    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
-    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
-    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+    SPDX-License-Identifier: MIT
 */
 
-
 #include "definition.h"
 #include "definition_p.h"
 #include "definitionref_p.h"
@@ -31,20 +14,19 @@
 #include "context_p.h"
 #include "format.h"
 #include "format_p.h"
+#include "ksyntaxhighlighting_logging.h"
+#include "ksyntaxhighlighting_version.h"
 #include "repository.h"
 #include "repository_p.h"
 #include "rule_p.h"
-#include "ksyntaxhighlighting_logging.h"
-#include "ksyntaxhighlighting_version.h"
+#include "worddelimiters_p.h"
 #include "xml_p.h"
 
+#include <QCborMap>
 #include <QCoreApplication>
-#include <QDebug>
 #include <QFile>
 #include <QHash>
-#include <QJsonObject>
 #include <QStringList>
-#include <QVector>
 #include <QXmlStreamReader>
 
 #include <algorithm>
@@ -52,7 +34,7 @@
 using namespace KSyntaxHighlighting;
 
 DefinitionData::DefinitionData()
-    : wordDelimiters(QStringLiteral("\t !%&()*+,-./:;<=>?[\\]^{|}~")) // must be sorted!
+    : wordDelimiters()
     , wordWrapDelimiters(wordDelimiters)
 {
 }
@@ -62,24 +44,24 @@ DefinitionData::~DefinitionData()
     qDeleteAll(contexts);
 }
 
-DefinitionData* DefinitionData::get(const Definition &def)
+DefinitionData *DefinitionData::get(const Definition &def)
 {
     return def.d.get();
 }
 
-Definition::Definition() :
-    d(new DefinitionData)
+Definition::Definition()
+    : d(new DefinitionData)
 {
 }
 
-Definition::Definition(const Definition &other) :
-    d(other.d)
+Definition::Definition(const Definition &other)
+    : d(other.d)
 {
     d->q = *this;
 }
 
-Definition::Definition(const std::shared_ptr<DefinitionData> &dd) :
-    d(dd)
+Definition::Definition(std::shared_ptr<DefinitionData> &&dd)
+    : d(std::move(dd))
 {
 }
 
@@ -87,7 +69,7 @@ Definition::~Definition()
 {
 }
 
-Definition& Definition::operator=(const Definition &rhs)
+Definition &Definition::operator=(const Definition &rhs)
 {
     d = rhs.d;
     return *this;
@@ -98,7 +80,7 @@ bool Definition::operator==(const Definition &other) const
     return d->fileName == other.d->fileName;
 }
 
-bool Definition::operator!=(const Definition& other) const
+bool Definition::operator!=(const Definition &other) const
 {
     return d->fileName != other.d->fileName;
 }
@@ -181,13 +163,13 @@ QString Definition::license() const
 bool Definition::isWordDelimiter(QChar c) const
 {
     d->load();
-    return d->isWordDelimiter(c);
+    return d->wordDelimiters.contains(c);
 }
 
 bool Definition::isWordWrapDelimiter(QChar c) const
 {
     d->load();
-    return std::binary_search(d->wordWrapDelimiters.constBegin(), d->wordWrapDelimiters.constEnd(), c);
+    return d->wordWrapDelimiters.contains(c);
 }
 
 bool Definition::foldingEnabled() const
@@ -227,20 +209,31 @@ QStringList Definition::keywordLists() const
     return d->keywordLists.keys();
 }
 
-QStringList Definition::keywordList(const QString& name) const
+QStringList Definition::keywordList(const QString &name) const
 {
     d->load(DefinitionData::OnlyKeywords(true));
     const auto list = d->keywordList(name);
     return list ? list->keywords() : QStringList();
 }
 
+bool Definition::setKeywordList(const QString &name, const QStringList &content)
+{
+    d->load(DefinitionData::OnlyKeywords(true));
+    KeywordList *list = d->keywordList(name);
+    if (list) {
+        list->setKeywordList(content);
+        return true;
+    } else
+        return false;
+}
+
 QVector<Format> Definition::formats() const
 {
     d->load();
 
     // sort formats so that the order matches the order of the itemDatas in the xml files.
     auto formatList = QVector<Format>::fromList(d->formats.values());
-    std::sort(formatList.begin(), formatList.end(), [](const KSyntaxHighlighting::Format & lhs, const KSyntaxHighlighting::Format & rhs){
+    std::sort(formatList.begin(), formatList.end(), [](const KSyntaxHighlighting::Format &lhs, const KSyntaxHighlighting::Format &rhs) {
         return lhs.id() < rhs.id();
     });
 
@@ -258,9 +251,10 @@ QVector<Definition> Definition::includedDefinitions() const
         // Iterate all context rules to find associated Definitions. This will
         // automatically catch other Definitions referenced with IncludeRuldes or ContextSwitch.
         const auto definition = queue.takeLast();
-        for (const auto & context : qAsConst(definition.d->contexts)) {
+        for (const auto &context : qAsConst(definition.d->contexts)) {
             // handle context switch attributes of this context itself
-            for (const auto switchContext : {context->lineEndContext().context(), context->lineEmptyContext().context(), context->fallthroughContext().context()}) {
+            for (const auto switchContext :
+                 {context->lineEndContext().context(), context->lineEmptyContext().context(), context->fallthroughContext().context()}) {
                 if (switchContext) {
                     if (!definitions.contains(switchContext->definition())) {
                         queue.push_back(switchContext->definition());
@@ -309,7 +303,7 @@ CommentPosition Definition::singleLineCommentPosition() const
 QPair<QString, QString> Definition::multiLineCommentMarker() const
 {
     d->load();
-    return { d->multiLineCommentStartMarker, d->multiLineCommentEndMarker };
+    return {d->multiLineCommentStartMarker, d->multiLineCommentEndMarker};
 }
 
 QVector<QPair<QChar, QString>> Definition::characterEncodings() const
@@ -318,35 +312,30 @@ QVector<QPair<QChar, QString>> Definition::characterEncodings() const
     return d->characterEncodings;
 }
 
-Context* DefinitionData::initialContext() const
+Context *DefinitionData::initialContext() const
 {
     Q_ASSERT(!contexts.isEmpty());
     return contexts.first();
 }
 
-Context* DefinitionData::contextByName(const QString& name) const
+Context *DefinitionData::contextByName(const QString &wantedName) const
 {
-    foreach (auto context, contexts) {
-        if (context->name() == name)
+    for (const auto context : contexts) {
+        if (context->name() == wantedName)
             return context;
     }
     return nullptr;
 }
 
-KeywordList *DefinitionData::keywordList(const QString& name)
+KeywordList *DefinitionData::keywordList(const QString &wantedName)
 {
-    auto it = keywordLists.find(name);
+    auto it = keywordLists.find(wantedName);
     return (it == keywordLists.end()) ? nullptr : &it.value();
 }
 
-bool DefinitionData::isWordDelimiter(QChar c) const
+Format DefinitionData::formatByName(const QString &wantedName) const
 {
-    return std::binary_search(wordDelimiters.constBegin(), wordDelimiters.constEnd(), c);
-}
-
-Format DefinitionData::formatByName(const QString& name) const
-{
-    const auto it = formats.constFind(name);
+    const auto it = formats.constFind(wantedName);
     if (it != formats.constEnd())
         return it.value();
 
@@ -394,13 +383,18 @@ bool DefinitionData::load(OnlyKeywords onlyKeywords)
         it->setCaseSensitivity(caseSensitive);
     }
 
-    foreach (auto context, contexts) {
+    for (const auto context : qAsConst(contexts)) {
         context->resolveContexts();
         context->resolveIncludes();
         context->resolveAttributeFormat();
     }
 
-    Q_ASSERT(std::is_sorted(wordDelimiters.constBegin(), wordDelimiters.constEnd()));
+    for (const auto context : qAsConst(contexts)) {
+        for (const auto &rule : context->rules()) {
+            rule->resolvePostProcessing();
+        }
+    }
+
     return true;
 }
 
@@ -420,7 +414,7 @@ void DefinitionData::clear()
     license.clear();
     mimetypes.clear();
     extensions.clear();
-    wordDelimiters = QStringLiteral("\t !%&()*+,-./:;<=>?[\\]^{|}~"); // must be sorted!
+    wordDelimiters = WordDelimiters();
     wordWrapDelimiters = wordDelimiters;
     caseSensitive = Qt::CaseSensitive;
     version = 0.0f;
@@ -428,7 +422,7 @@ void DefinitionData::clear()
     hidden = false;
 }
 
-bool DefinitionData::loadMetaData(const QString& definitionFileName)
+bool DefinitionData::loadMetaData(const QString &definitionFileName)
 {
     fileName = definitionFileName;
 
@@ -449,24 +443,24 @@ bool DefinitionData::loadMetaData(const QString& definitionFileName)
     return false;
 }
 
-bool DefinitionData::loadMetaData(const QString &file, const QJsonObject &obj)
+bool DefinitionData::loadMetaData(const QString &file, const QCborMap &obj)
 {
-    name     = obj.value(QLatin1String("name")).toString();
-    section  = obj.value(QLatin1String("section")).toString();
-    version  = obj.value(QLatin1String("version")).toInt();
-    priority = obj.value(QLatin1String("priority")).toInt();
-    style    = obj.value(QLatin1String("style")).toString();
-    author   = obj.value(QLatin1String("author")).toString();
-    license  = obj.value(QLatin1String("license")).toString();
+    name = obj.value(QLatin1String("name")).toString();
+    section = obj.value(QLatin1String("section")).toString();
+    version = obj.value(QLatin1String("version")).toInteger();
+    priority = obj.value(QLatin1String("priority")).toInteger();
+    style = obj.value(QLatin1String("style")).toString();
+    author = obj.value(QLatin1String("author")).toString();
+    license = obj.value(QLatin1String("license")).toString();
     indenter = obj.value(QLatin1String("indenter")).toString();
-    hidden   = obj.value(QLatin1String("hidden")).toBool();
+    hidden = obj.value(QLatin1String("hidden")).toBool();
     fileName = file;
 
     const auto exts = obj.value(QLatin1String("extensions")).toString();
-    foreach (const auto &ext, exts.split(QLatin1Char(';'), QString::SkipEmptyParts))
+    for (const auto &ext : exts.split(QLatin1Char(';'), Qt::SkipEmptyParts))
         extensions.push_back(ext);
     const auto mts = obj.value(QLatin1String("mimetype")).toString();
-    foreach (const auto &mt, mts.split(QLatin1Char(';'), QString::SkipEmptyParts))
+    for (const auto &mt : mts.split(QLatin1Char(';'), Qt::SkipEmptyParts))
         mimetypes.push_back(mt);
 
     return true;
@@ -477,31 +471,31 @@ bool DefinitionData::loadLanguage(QXmlStreamReader &reader)
     Q_ASSERT(reader.name() == QLatin1String("language"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
 
-    if (!checkKateVersion(reader.attributes().value(QStringLiteral("kateversion"))))
+    if (!checkKateVersion(reader.attributes().value(QLatin1String("kateversion"))))
         return false;
 
-    name = reader.attributes().value(QStringLiteral("name")).toString();
-    section = reader.attributes().value(QStringLiteral("section")).toString();
+    name = reader.attributes().value(QLatin1String("name")).toString();
+    section = reader.attributes().value(QLatin1String("section")).toString();
     // toFloat instead of toInt for backward compatibility with old Kate files
-    version = reader.attributes().value(QStringLiteral("version")).toFloat();
-    priority = reader.attributes().value(QStringLiteral("priority")).toInt();
-    hidden = Xml::attrToBool(reader.attributes().value(QStringLiteral("hidden")));
-    style = reader.attributes().value(QStringLiteral("style")).toString();
-    indenter = reader.attributes().value(QStringLiteral("indenter")).toString();
-    author = reader.attributes().value(QStringLiteral("author")).toString();
-    license = reader.attributes().value(QStringLiteral("license")).toString();
-    const auto exts = reader.attributes().value(QStringLiteral("extensions")).toString();
-    foreach (const auto &ext, exts.split(QLatin1Char(';'), QString::SkipEmptyParts))
+    version = reader.attributes().value(QLatin1String("version")).toFloat();
+    priority = reader.attributes().value(QLatin1String("priority")).toInt();
+    hidden = Xml::attrToBool(reader.attributes().value(QLatin1String("hidden")));
+    style = reader.attributes().value(QLatin1String("style")).toString();
+    indenter = reader.attributes().value(QLatin1String("indenter")).toString();
+    author = reader.attributes().value(QLatin1String("author")).toString();
+    license = reader.attributes().value(QLatin1String("license")).toString();
+    const auto exts = reader.attributes().value(QLatin1String("extensions")).toString();
+    for (const auto &ext : exts.split(QLatin1Char(';'), Qt::SkipEmptyParts))
         extensions.push_back(ext);
-    const auto mts = reader.attributes().value(QStringLiteral("mimetype")).toString();
-    foreach (const auto &mt, mts.split(QLatin1Char(';'), QString::SkipEmptyParts))
+    const auto mts = reader.attributes().value(QLatin1String("mimetype")).toString();
+    for (const auto &mt : mts.split(QLatin1Char(';'), Qt::SkipEmptyParts))
         mimetypes.push_back(mt);
-    if (reader.attributes().hasAttribute(QStringLiteral("casesensitive")))
-        caseSensitive = Xml::attrToBool(reader.attributes().value(QStringLiteral("casesensitive"))) ? Qt::CaseSensitive : Qt::CaseInsensitive;
+    if (reader.attributes().hasAttribute(QLatin1String("casesensitive")))
+        caseSensitive = Xml::attrToBool(reader.attributes().value(QLatin1String("casesensitive"))) ? Qt::CaseSensitive : Qt::CaseInsensitive;
     return true;
 }
 
-void DefinitionData::loadHighlighting(QXmlStreamReader& reader, OnlyKeywords onlyKeywords)
+void DefinitionData::loadHighlighting(QXmlStreamReader &reader, OnlyKeywords onlyKeywords)
 {
     Q_ASSERT(reader.name() == QLatin1String("highlighting"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
@@ -511,34 +505,34 @@ void DefinitionData::loadHighlighting(QXmlStreamReader& reader, OnlyKeywords onl
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                if (reader.name() == QLatin1String("list")) {
-                    if (!keywordIsLoaded) {
-                        KeywordList keywords;
-                        keywords.load(reader);
-                        keywordLists.insert(keywords.name(), keywords);
-                    }
-                    else {
-                        reader.skipCurrentElement();
-                    }
-                } else if (bool(onlyKeywords)) {
-                    resolveIncludeKeywords();
-                    return;
-                } else if (reader.name() == QLatin1String("contexts")) {
-                    resolveIncludeKeywords();
-                    loadContexts(reader);
-                    reader.readNext();
-                } else if (reader.name() == QLatin1String("itemDatas")) {
-                    loadItemData(reader);
+        case QXmlStreamReader::StartElement:
+            if (reader.name() == QLatin1String("list")) {
+                if (!keywordIsLoaded) {
+                    KeywordList keywords;
+                    keywords.load(reader);
+                    keywordLists.insert(keywords.name(), keywords);
                 } else {
-                    reader.readNext();
+                    reader.skipCurrentElement();
+                    reader.readNext(); // Skip </list>
                 }
-                break;
-            case QXmlStreamReader::EndElement:
+            } else if (bool(onlyKeywords)) {
+                resolveIncludeKeywords();
                 return;
-            default:
+            } else if (reader.name() == QLatin1String("contexts")) {
+                resolveIncludeKeywords();
+                loadContexts(reader);
                 reader.readNext();
-                break;
+            } else if (reader.name() == QLatin1String("itemDatas")) {
+                loadItemData(reader);
+            } else {
+                reader.readNext();
+            }
+            break;
+        case QXmlStreamReader::EndElement:
+            return;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
@@ -556,60 +550,60 @@ void DefinitionData::resolveIncludeKeywords()
     }
 }
 
-void DefinitionData::loadContexts(QXmlStreamReader& reader)
+void DefinitionData::loadContexts(QXmlStreamReader &reader)
 {
     Q_ASSERT(reader.name() == QLatin1String("contexts"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                if (reader.name() == QLatin1String("context")) {
-                    auto context = new Context;
-                    context->setDefinition(q);
-                    context->load(reader);
-                    contexts.push_back(context);
-                }
-                reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                return;
-            default:
-                reader.readNext();
-                break;
+        case QXmlStreamReader::StartElement:
+            if (reader.name() == QLatin1String("context")) {
+                auto context = new Context;
+                context->setDefinition(q);
+                context->load(reader);
+                contexts.push_back(context);
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            return;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
 
-void DefinitionData::loadItemData(QXmlStreamReader& reader)
+void DefinitionData::loadItemData(QXmlStreamReader &reader)
 {
     Q_ASSERT(reader.name() == QLatin1String("itemDatas"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                if (reader.name() == QLatin1String("itemData")) {
-                    Format f;
-                    auto formatData = FormatPrivate::detachAndGet(f);
-                    formatData->definition = q;
-                    formatData->load(reader);
-                    formatData->id = RepositoryPrivate::get(repo)->nextFormatId();
-                    formats.insert(f.name(), f);
-                    reader.readNext();
-                }
+        case QXmlStreamReader::StartElement:
+            if (reader.name() == QLatin1String("itemData")) {
+                Format f;
+                auto formatData = FormatPrivate::detachAndGet(f);
+                formatData->definition = q;
+                formatData->load(reader);
+                formatData->id = RepositoryPrivate::get(repo)->nextFormatId();
+                formats.insert(f.name(), f);
                 reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                return;
-            default:
-                reader.readNext();
-                break;
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            return;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
 
-void DefinitionData::loadGeneral(QXmlStreamReader& reader)
+void DefinitionData::loadGeneral(QXmlStreamReader &reader)
 {
     Q_ASSERT(reader.name() == QLatin1String("general"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
@@ -620,49 +614,48 @@ void DefinitionData::loadGeneral(QXmlStreamReader& reader)
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                ++elementRefCounter;
+        case QXmlStreamReader::StartElement:
+            ++elementRefCounter;
 
-                if (reader.name() == QLatin1String("keywords")) {
-                    if (reader.attributes().hasAttribute(QStringLiteral("casesensitive")))
-                        caseSensitive = Xml::attrToBool(reader.attributes().value(QStringLiteral("casesensitive"))) ? Qt::CaseSensitive : Qt::CaseInsensitive;
+            if (reader.name() == QLatin1String("keywords")) {
+                if (reader.attributes().hasAttribute(QLatin1String("casesensitive")))
+                    caseSensitive = Xml::attrToBool(reader.attributes().value(QLatin1String("casesensitive"))) ? Qt::CaseSensitive : Qt::CaseInsensitive;
 
-                    // adapt sorted wordDelimiters
-                    wordDelimiters += reader.attributes().value(QStringLiteral("additionalDeliminator"));
-                    std::sort(wordDelimiters.begin(), wordDelimiters.end());
-                    auto it = std::unique(wordDelimiters.begin(), wordDelimiters.end());
-                    wordDelimiters.truncate(std::distance(wordDelimiters.begin(), it));
-                    foreach (const auto c, reader.attributes().value(QLatin1String("weakDeliminator")))
-                        wordDelimiters.remove(c);
+                // adapt wordDelimiters
+                wordDelimiters.append(reader.attributes().value(QLatin1String("additionalDeliminator")));
+                wordDelimiters.remove(reader.attributes().value(QLatin1String("weakDeliminator")));
 
-                    // adaptWordWrapDelimiters, and sort
-                    wordWrapDelimiters = reader.attributes().value(QStringLiteral("wordWrapDeliminator")).toString();
-                    std::sort(wordWrapDelimiters.begin(), wordWrapDelimiters.end());
-                    if (wordWrapDelimiters.isEmpty())
-                        wordWrapDelimiters = wordDelimiters;
-                } else if (reader.name() == QLatin1String("folding")) {
-                    if (reader.attributes().hasAttribute(QStringLiteral("indentationsensitive")))
-                        indentationBasedFolding = Xml::attrToBool(reader.attributes().value(QStringLiteral("indentationsensitive")));
-                } else if (reader.name() == QLatin1String("emptyLines")) {
-                    loadFoldingIgnoreList(reader);
-                } else if (reader.name() == QLatin1String("comments")) {
-                    loadComments(reader);
-                } else if (reader.name() == QLatin1String("spellchecking")) {
-                    loadSpellchecking(reader);
-                } else {
-                    reader.skipCurrentElement();
+                // adapt WordWrapDelimiters
+                auto wordWrapDeliminatorAttr = reader.attributes().value(
+                    QLatin1String("wordWrapDeliminator"));
+                if (wordWrapDeliminatorAttr.isEmpty())
+                    wordWrapDelimiters = wordDelimiters;
+                else {
+                    wordWrapDelimiters.append(wordWrapDeliminatorAttr);
                 }
-                reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                --elementRefCounter;
-                if (elementRefCounter == 0)
-                    return;
-                reader.readNext();
-                break;
-            default:
-                reader.readNext();
-                break;
+            } else if (reader.name() == QLatin1String("folding")) {
+                if (reader.attributes().hasAttribute(QLatin1String("indentationsensitive")))
+                    indentationBasedFolding = Xml::attrToBool(reader.attributes().value(QLatin1String("indentationsensitive")));
+            } else if (reader.name() == QLatin1String("emptyLines")) {
+                loadFoldingIgnoreList(reader);
+            } else if (reader.name() == QLatin1String("comments")) {
+                loadComments(reader);
+            } else if (reader.name() == QLatin1String("spellchecking")) {
+                loadSpellchecking(reader);
+            } else {
+                reader.skipCurrentElement();
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            --elementRefCounter;
+            if (elementRefCounter == 0)
+                return;
+            reader.readNext();
+            break;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
@@ -678,35 +671,35 @@ void DefinitionData::loadComments(QXmlStreamReader &reader)
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                ++elementRefCounter;
-                if (reader.name() == QLatin1String("comment")) {
-                    const bool isSingleLine = reader.attributes().value(QStringLiteral("name")) == QStringLiteral("singleLine");
-                    if (isSingleLine) {
-                        singleLineCommentMarker = reader.attributes().value(QStringLiteral("start")).toString();
-                        const bool afterWhiteSpace = reader.attributes().value(QStringLiteral("position")).toString() == QStringLiteral("afterwhitespace");
-                        singleLineCommentPosition = afterWhiteSpace ? CommentPosition::AfterWhitespace : CommentPosition::StartOfLine;
-                    } else {
-                        multiLineCommentStartMarker = reader.attributes().value(QStringLiteral("start")).toString();
-                        multiLineCommentEndMarker = reader.attributes().value(QStringLiteral("end")).toString();
-                    }
+        case QXmlStreamReader::StartElement:
+            ++elementRefCounter;
+            if (reader.name() == QLatin1String("comment")) {
+                const bool isSingleLine = reader.attributes().value(QLatin1String("name")) == QLatin1String("singleLine");
+                if (isSingleLine) {
+                    singleLineCommentMarker = reader.attributes().value(QLatin1String("start")).toString();
+                    const bool afterWhiteSpace = reader.attributes().value(QLatin1String("position")).toString() == QLatin1String("afterwhitespace");
+                    singleLineCommentPosition = afterWhiteSpace ? CommentPosition::AfterWhitespace : CommentPosition::StartOfLine;
+                } else {
+                    multiLineCommentStartMarker = reader.attributes().value(QLatin1String("start")).toString();
+                    multiLineCommentEndMarker = reader.attributes().value(QLatin1String("end")).toString();
                 }
-                reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                --elementRefCounter;
-                if (elementRefCounter == 0)
-                    return;
-                reader.readNext();
-                break;
-            default:
-                reader.readNext();
-                break;
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            --elementRefCounter;
+            if (elementRefCounter == 0)
+                return;
+            reader.readNext();
+            break;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
 
-void DefinitionData::loadFoldingIgnoreList(QXmlStreamReader& reader)
+void DefinitionData::loadFoldingIgnoreList(QXmlStreamReader &reader)
 {
     Q_ASSERT(reader.name() == QLatin1String("emptyLines"));
     Q_ASSERT(reader.tokenType() == QXmlStreamReader::StartElement);
@@ -717,22 +710,22 @@ void DefinitionData::loadFoldingIgnoreList(QXmlStreamReader& reader)
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                ++elementRefCounter;
-                if (reader.name() == QLatin1String("emptyLine")) {
-                    foldingIgnoreList << reader.attributes().value(QStringLiteral("regexpr")).toString();
-                }
-                reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                --elementRefCounter;
-                if (elementRefCounter == 0)
-                    return;
-                reader.readNext();
-                break;
-            default:
-                reader.readNext();
-                break;
+        case QXmlStreamReader::StartElement:
+            ++elementRefCounter;
+            if (reader.name() == QLatin1String("emptyLine")) {
+                foldingIgnoreList << reader.attributes().value(QLatin1String("regexpr")).toString();
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            --elementRefCounter;
+            if (elementRefCounter == 0)
+                return;
+            reader.readNext();
+            break;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
@@ -748,39 +741,39 @@ void DefinitionData::loadSpellchecking(QXmlStreamReader &reader)
 
     while (!reader.atEnd()) {
         switch (reader.tokenType()) {
-            case QXmlStreamReader::StartElement:
-                ++elementRefCounter;
-                if (reader.name() == QLatin1String("encoding")) {
-                    const auto charRef = reader.attributes().value(QStringLiteral("char"));
-                    if (!charRef.isEmpty()) {
-                        const auto str = reader.attributes().value(QStringLiteral("string")).toString();
-                        characterEncodings.push_back({ charRef[0], str });
-                    }
+        case QXmlStreamReader::StartElement:
+            ++elementRefCounter;
+            if (reader.name() == QLatin1String("encoding")) {
+                const auto charRef = reader.attributes().value(QLatin1String("char"));
+                if (!charRef.isEmpty()) {
+                    const auto str = reader.attributes().value(QLatin1String("string")).toString();
+                    characterEncodings.push_back({charRef[0], str});
                 }
-                reader.readNext();
-                break;
-            case QXmlStreamReader::EndElement:
-                --elementRefCounter;
-                if (elementRefCounter == 0)
-                    return;
-                reader.readNext();
-                break;
-            default:
-                reader.readNext();
-                break;
+            }
+            reader.readNext();
+            break;
+        case QXmlStreamReader::EndElement:
+            --elementRefCounter;
+            if (elementRefCounter == 0)
+                return;
+            reader.readNext();
+            break;
+        default:
+            reader.readNext();
+            break;
         }
     }
 }
 
-bool DefinitionData::checkKateVersion(const QStringRef& verStr)
+bool DefinitionData::checkKateVersion(QStringView verStr)
 {
     const auto idx = verStr.indexOf(QLatin1Char('.'));
     if (idx <= 0) {
         qCWarning(Log) << "Skipping" << fileName << "due to having no valid kateversion attribute:" << verStr;
         return false;
     }
-    const auto major = verStr.left(idx).toInt();
-    const auto minor = verStr.mid(idx + 1).toInt();
+    const auto major = verStr.left(idx).toString().toInt();
+    const auto minor = verStr.mid(idx + 1).toString().toInt();
 
     if (major > SyntaxHighlighting_VERSION_MAJOR || (major == SyntaxHighlighting_VERSION_MAJOR && minor > SyntaxHighlighting_VERSION_MINOR)) {
         qCWarning(Log) << "Skipping" << fileName << "due to being too new, version:" << verStr;
@@ -800,8 +793,8 @@ DefinitionRef::DefinitionRef()
 {
 }
 
-DefinitionRef::DefinitionRef(const Definition &def) :
-    d(def.d)
+DefinitionRef::DefinitionRef(const Definition &def)
+    : d(def.d)
 {
 }
 
@@ -809,7 +802,7 @@ DefinitionRef::~DefinitionRef()
 {
 }
 
-DefinitionRef& DefinitionRef::operator=(const Definition &def)
+DefinitionRef &DefinitionRef::operator=(const Definition &def)
 {
     d = def.d;
     return *this;
